// 排版
import 'package:desktop_ui/constants.dart';
import 'package:desktop_ui/enums/common.dart';
import 'package:desktop_ui/utils/text.dart';
import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher.dart';

class TypographyBase extends StatelessWidget {
  const TypographyBase(
    this.title, {
    Key? key,
    required this.level,
    this.type,
    this.italic,
    this.disable,
    this.underline,
    this.delete,
    this.link,
    this.strong,
    this.mark,
    this.code,
    this.ellipsis,
    this.maxLines,
  }) : super(key: key);

  /// 标题内容
  final String title;

  /// 级别
  final EasyTextLevel level;

  /// 颜色主题类型
  final EasyThemeType? type;

  /// 是否是斜体
  final bool? italic;

  /// 是否禁用
  final bool? disable;

  /// 是否有下划线
  final bool? underline;

  /// 是否删除线
  final bool? delete;

  /// 是否链接
  final bool? link;

  /// 是否加粗
  final bool? strong;

  /// 是否是标记
  final bool? mark;

  /// 是否是代码
  final bool? code;

  /// 是否自动省略
  final bool? ellipsis;

  /// 最多显示多少行
  final int? maxLines;

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    bool isCode = code != null && code!;
    bool isContent = level == EasyTextLevel.content;
    return Container(
      constraints: BoxConstraints(
        minWidth: isContent ? 0 : size.width,
      ),
      padding: EdgeInsets.only(
        left: isCode ? iDefaultGap : 0,
        right: isCode ? iDefaultGap : 0,
        top: isContent ? 3 : iDefaultGap,
        bottom: isContent ? 3 : iDefaultGap,
      ),
      decoration: BoxDecoration(
        border: isCode ? iBorder : iBorderNone,
        color: isCode ? iGrayLight.withOpacity(.6) : iTransparent,
        borderRadius: isCode ? iBorderRadiusSmall : iBorderRadiusNone,
      ),
      child: Text(
        title,
        overflow: getTextOverflow(ellipsis),
        maxLines: maxLines ?? 999999,
        style: TextStyle(
          fontSize: getFontSize(level),
          fontWeight: getFontWeight(strong),
          color: getTextColor(type, disable, mark),
          fontStyle: getFontStyle(italic),
          decoration: getTextDecoration(underline, delete),
          backgroundColor: getTextBackgroundColor(type, mark),
        ),
      ),
    );
  }
}

class EasyTitle extends StatelessWidget {
  /// 标题，支持 h1-h5 样式，默认：h1
  const EasyTitle(
    this.title, {
    Key? key,
    this.level,
    this.type,
    this.italic,
    this.disable,
    this.underline,
    this.delete,
  }) : super(key: key);

  /// 标题内容
  final String title;

  /// 级别
  final EasyTextLevel? level;

  /// 颜色主题类型
  final EasyThemeType? type;

  /// 是否是斜体
  final bool? italic;

  /// 是否禁用
  final bool? disable;

  /// 是否有下划线
  final bool? underline;

  /// 是否删除线
  final bool? delete;

  @override
  Widget build(BuildContext context) {
    return TypographyBase(
      title,
      level: level ?? EasyTextLevel.h1,
      type: type,
      italic: italic,
      disable: disable,
      underline: underline,
      delete: delete,
    );
  }
}

class EasyText extends StatelessWidget {
  /// 文本
  ///
  /// 支持删除、标记、加粗、斜体、下划线
  const EasyText(
    this.text, {
    Key? key,
    this.type,
    this.italic,
    this.disable,
    this.underline,
    this.delete,
    this.strong,
    this.mark,
    this.code,
  }) : super(key: key);

  /// 标题内容
  final String text;

  /// 颜色主题类型
  final EasyThemeType? type;

  /// 是否是斜体
  final bool? italic;

  /// 是否禁用
  final bool? disable;

  /// 是否有下划线
  final bool? underline;

  /// 是否删除线
  final bool? delete;

  /// 是否加粗
  final bool? strong;

  /// 是否是标记
  final bool? mark;

  /// 是否是代码
  final bool? code;

  @override
  Widget build(BuildContext context) {
    return TypographyBase(
      text,
      level: EasyTextLevel.content,
      type: type,
      italic: italic,
      disable: disable,
      underline: underline,
      delete: delete,
      strong: strong,
      mark: mark,
      code: code,
    );
  }
}

class EasyParagraph extends StatelessWidget {
  /// 自然段
  ///
  /// 支持设置最大行数，自动省略号
  const EasyParagraph(this.content, {Key? key, this.ellipsis, this.maxLines}) : super(key: key);

  /// 标题内容
  final String content;

  /// 是否自动省略
  final bool? ellipsis;

  /// 最多显示多少行
  final int? maxLines;

  @override
  Widget build(BuildContext context) {
    return TypographyBase(
      content,
      level: EasyTextLevel.paragraph,
      ellipsis: ellipsis,
      maxLines: maxLines,
    );
  }
}

class EasyLink extends StatelessWidget {
  /// 链接
  const EasyLink(this.title, this.link, {Key? key}) : super(key: key);

  /// 链接标题
  final String title;

  /// 链接地址
  final String link;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () async {
        if (!await launch(link)) throw '不能打开链接 $link';
      },
      child: Text(
        title,
        style: TextStyle(
          fontSize: getFontSize(EasyTextLevel.content),
          color: iPrimary,
        ),
      ),
    );
  }
}
